﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Follower : MonoBehaviour
{
    public Transform target;

    public bool following = false;
    public AnimationCurve forceCurve;
    public float forceCoef = 1f;
    public float cutoffThres = 3f;
    public float followingTime = 0f;
    public float maxFollowTime = 6f;
    public bool activated = false;

    public new Rigidbody rigidbody { get { return GetComponent<Rigidbody>(); } }

    public AudioSource audio { get { return GetComponent<AudioSource>(); } }

    // Start is called before the first frame update
    void Start()
    {
        target = GameObject.FindGameObjectWithTag("Player").transform;
    }

    // Update is called once per frame
    void Update()
    {
        if (following) {
            Vector3 relPos = target.position - transform.position;
            float d = relPos.magnitude;
            rigidbody.AddForce(forceCurve.Evaluate(d) * forceCoef / d * relPos, ForceMode.Impulse);
            if (followingTime > 3f && d > cutoffThres) following = false;
            if (followingTime > maxFollowTime) following = false;
            followingTime += Time.deltaTime;
        }

        // Rotate
        Vector3 axis = Vector3.Cross(rigidbody.velocity, Vector3.forward);
        transform.Rotate(axis, Time.deltaTime * rigidbody.velocity.magnitude / transform.localScale.x * 2f * Mathf.Rad2Deg, Space.World);
    }

    IEnumerator BarkCoroutine() {
        if (following) {
            yield return new WaitForSeconds(Random.Range(0.3f, 3f));
        }
    }

    private void OnTriggerEnter(Collider other) {
        if (other.tag == "Player") {
            if (!activated) {
                activated = true;
                following = true;
                GetComponent<SphereCollider>().enabled = true;
                GetComponent<MeshRenderer>().enabled = true;
                GetComponent<AudioControl>().PlayRandom();
                StartCoroutine(BarkCoroutine());
            }
        }
    }
}
