﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerControl : MonoBehaviour {

    public bool gameover = false;
    [SerializeField] AudioSource bgm;

    public float pushForce = 0.05f;
    public float forceCoef = 1f;
    public Transform cameraContainer;
    public Camera mainCamera;

    public int healthMax = 3;
    public int health;
    [Range(0f, 1f)] public float energy = 1f;
    public Vector2 goodEnergyRange = new Vector2(0.3f, 0.8f);
    public float energyReduceFactor = 0.03f;

    public HealthBar healthBar;
    public EnergyBar energyBar;
    
    private new Rigidbody rigidbody { get { return GetComponent<Rigidbody>(); } }

    public static void VisualizeForce(Vector3 target, Vector3 f, Color color) {
        Debug.DrawLine(target, target + f * 2, color, Time.deltaTime, false);
    }

    // Use this for initialization
    void Start () {
        health = healthMax;
        energy = 1f;
        healthBar.SetHealth(health, healthMax);
        energyBar.SetEnergy(energy);
	}
	
	// Update is called once per frame
	void Update () {
        // update camera
        Vector3 ccPos = cameraContainer.position;
        if (transform.position.y > ccPos.y) ccPos.y = transform.position.y;
        cameraContainer.position = ccPos;

        // input
        if (Input.touchCount > 0) {
            Touch t = Input.touches[0];
            Vector3 focus = mainCamera.ScreenToWorldPoint(t.position);
            //Vector3 force = focus - transform.position;
            //GetComponent<Rigidbody>().AddForce(force, ForceMode.Impulse);
            //PlayerControl.VisualizeForce(transform.position, force);
            //Vector3 newPos = transform.position;
            //newPos.x = focus.x;
            //transform.position = newPos;
            Vector3 force = pushForce * forceCoef * t.deltaPosition;
            VisualizeForce(transform.position, force, Color.green);
            rigidbody.AddForce(force, ForceMode.Impulse);
        }

        // energy
        if (energy > 0f) energy -= Time.deltaTime * energyReduceFactor;
        if (energy < goodEnergyRange.x) forceCoef = 0.5f;
        else if (energy > goodEnergyRange.y) forceCoef = 0.5f;
        else forceCoef = 1f;

        // Rotate
        Vector3 axis = Vector3.Cross(rigidbody.velocity, Vector3.forward);
        transform.Rotate(axis, Time.deltaTime * rigidbody.velocity.magnitude / transform.localScale.x * 2f * Mathf.Rad2Deg, Space.World);

        // UI
        energyBar.SetEnergy(energy);
        healthBar.SetHealth(health, healthMax);

        // gameover
        if (!gameover) {
            if (health <= 0 || energy < 0f) {
                bgm.Stop();
                GetComponent<AudioControl>().PlayRandom();
                Time.timeScale = 0f;
            }
        }
        

        /*Vector3 f = Vector3.zero;
		if (Input.GetKey(KeyCode.W)) {
            f += pushForce * Vector3.up;
        }
        if (Input.GetKey(KeyCode.S)) {
            f += pushForce * Vector3.down;
        }
        if (Input.GetKey(KeyCode.A)) {
            f += pushForce * Vector3.left;
        }
        if (Input.GetKey(KeyCode.D)) {
            f += pushForce * Vector3.right;
        }

        VisualizeForce(transform.position, f, Color.green);
        rigidbody.AddForce(f, ForceMode.Impulse);*/
    }

    private void OnTriggerEnter(Collider other) {
        if (other.tag == "Food") {
            energy += other.GetComponent<Food>().energyProvided;
            //if (health < healthMax) health++;
            other.GetComponent<Food>().DestroyThis();
        } else if (other.tag == "Trash") {
            energy += other.GetComponent<Food>().energyProvided;
            if (health > 0) health--;
            other.GetComponent<Food>().DestroyThis();
        }
    }

    private void OnCollisionEnter(Collision collision) {
        if (collision.collider.tag == "Dog") {
            health--;
        }
    }
}
